require 'rails/generators/base'

module Revelry
  module Generators
    module Auth
      class InstallGenerator < Rails::Generators::Base
        source_root File.expand_path("../templates", __FILE__)

        def add_gem
          gem 'devise'
          gem 'rolify'
          gem 'pundit'
          gem 'devise_invitable'
          run 'bundle install'
        end

        def install_devise
          generate 'devise:install'
        end

        def install_devise_to_user
          generate 'devise user'
          insert_into_file Rails.root.join('app', 'models', 'user.rb'), after: 'validatable' do
            ', :invitable'
          end
          rake 'db:migrate'
        end

        def install_roles
          generate 'rolify Role User'
          rake 'db:migrate'
        end

        def add_pundit_to_app_controller
          insert_into_file app_controller_path, after: "ActionController::Base\n" do
            "include Pundit\n"
          end
        end

        def install_policies
          generate 'pundit:install'
          template 'admins_only_policy.rb', 'app/policies/admins_only_policy.rb'
        end

        private

        def app_controller_path
          Rails.root.join('app', 'controllers', 'application_controller.rb')
        end
      end
    end
  end
end
